import * as __WEBPACK_EXTERNAL_MODULE_kysely__ from "kysely";
import * as __WEBPACK_EXTERNAL_MODULE_pg__ from "pg";
const postgres = (config, hooks)=>(_)=>{
        const pool = new __WEBPACK_EXTERNAL_MODULE_pg__.Pool(config);
        const dialect = new __WEBPACK_EXTERNAL_MODULE_kysely__.PostgresDialect({
            pool
        });
        const db = new __WEBPACK_EXTERNAL_MODULE_kysely__.Kysely({
            dialect
        });
        let bundles = [];
        const changedIds = new Set();
        function markChanged(id) {
            changedIds.add(id);
        }
        let isUnmount = false;
        return {
            name: "postgres",
            async onUnmount () {
                if (isUnmount) return;
                isUnmount = true;
                await pool.end();
                changedIds.clear();
            },
            async commitBundle () {
                if (0 === changedIds.size) return;
                const changedBundles = bundles.filter((b)=>changedIds.has(b.id));
                if (0 === changedBundles.length) return;
                await db.transaction().execute(async (tx)=>{
                    for (const bundle of changedBundles)await tx.insertInto("bundles").values({
                        id: bundle.id,
                        enabled: bundle.enabled,
                        file_url: bundle.fileUrl,
                        should_force_update: bundle.shouldForceUpdate,
                        file_hash: bundle.fileHash,
                        git_commit_hash: bundle.gitCommitHash,
                        message: bundle.message,
                        platform: bundle.platform,
                        target_app_version: bundle.targetAppVersion
                    }).onConflict((oc)=>oc.column("id").doUpdateSet({
                            enabled: bundle.enabled,
                            file_url: bundle.fileUrl,
                            should_force_update: bundle.shouldForceUpdate,
                            file_hash: bundle.fileHash,
                            git_commit_hash: bundle.gitCommitHash,
                            message: bundle.message,
                            platform: bundle.platform,
                            target_app_version: bundle.targetAppVersion
                        })).execute();
                });
                changedIds.clear();
                hooks?.onDatabaseUpdated?.();
            },
            async updateBundle (targetBundleId, newBundle) {
                bundles = await this.getBundles();
                const targetIndex = bundles.findIndex((u)=>u.id === targetBundleId);
                if (-1 === targetIndex) throw new Error("target bundle version not found");
                Object.assign(bundles[targetIndex], newBundle);
                markChanged(targetBundleId);
            },
            async appendBundle (inputBundle) {
                bundles = await this.getBundles();
                bundles.unshift(inputBundle);
                markChanged(inputBundle.id);
            },
            async getBundleById (bundleId) {
                const data = await db.selectFrom("bundles").selectAll().where("id", "=", bundleId).executeTakeFirst();
                if (!data) return null;
                return {
                    enabled: data.enabled,
                    fileUrl: data.file_url,
                    shouldForceUpdate: data.should_force_update,
                    fileHash: data.file_hash,
                    gitCommitHash: data.git_commit_hash,
                    id: data.id,
                    message: data.message,
                    platform: data.platform,
                    targetAppVersion: data.target_app_version
                };
            },
            async getBundles (refresh = false) {
                if (bundles.length > 0 && !refresh) return bundles;
                const data = await db.selectFrom("bundles").orderBy("id", "desc").selectAll().execute();
                return data.map((bundle)=>({
                        enabled: bundle.enabled,
                        fileUrl: bundle.file_url,
                        shouldForceUpdate: bundle.should_force_update,
                        fileHash: bundle.file_hash,
                        gitCommitHash: bundle.git_commit_hash,
                        id: bundle.id,
                        message: bundle.message,
                        platform: bundle.platform,
                        targetAppVersion: bundle.target_app_version
                    }));
            }
        };
    };
export { postgres };
